import { csv, select } from 'd3'; //import CSV from D3 
import { scatterPlot } from './scatterPlot'; //import scatterplot 

const csvUrl = [
  'https://gist.githubusercontent.com/anvesh2129/e78129b06abccfbc618b3d2d1dd1c130/raw/9cb082153bd6d03dfbff0bb3994b565773ee2aff/Iris.csv', // File name
].join(''); // passing the iris dataset URL
//Parsing the values to x-axis and y-axis
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

const width = window.innerWidth-60; // using D3 returning the interior width of the graph
const height = window.innerHeight-60; //using D3 returning the height of the graph
const svg = select('body') // Using D3 rendering the svg
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const xValue = (d) => d.petal_length;  // values in the x-axis of the scatter plot
const yValue = (d) => d.petal_width; //values in the y-axis of the scatter plo
const rxValue = (d) => d.sepal_length; //radius values in the x-axis of the scatter plot
const ryValue = (d) => d.sepal_width; //radius values in the y-axis of the scatter plot
const speciesValue = (d) => d.species; // Species values in scatter plot

// Data points in the scatter plot
const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const x = xValue((d) => d.petal_width);
  const y = yValue((d) => d.sepal_length);
 // defining colors to the species
  const marker = (d) => {
        	switch(d){ //using switch statement
            case 'setosa':
              return '#01796f';
            case 'versicolor':
              return '#000080';
            default:
              return '#ff4500';
          }
        }
  
// Define a marks array based on the data object, with x, y, species, color, rx, and ry properties.
const marks = data.map((d) => ({
  x: xValue((d) => d.petal_width),
  y: yValue((d) => d.sepal_length),
  species: speciesValue(d),
  color: marker(speciesValue(d)),
  rx: rxValue(d),
  ry: ryValue(d),
}));

// Define a scatter plot with the specified dimensions, data, x and y values, margins, and radius.
const plot = scatterPlot()
  .width(width)
  .height(height)
  .data(await csv(csvUrl, parseRow))
  .xValue((d) => d.petal_width)
  .yValue((d) => d.sepal_length)
  .margin({
    top: 34.5,
    right: 60,
    bottom: 79.5,
    left: 93,
  })
  .radius(4.5);

// Bind the marks to the SVG and add circles with the specified properties.
svg
  .selectAll('circle')
  .data(marks)
  .join('circle')
  .attr('cx', (d) => d.x)
  .attr('cy', (d) => d.y)
  .attr('fill', (d) => d.color)
  .attr('rx', (d) => (d.rx))
  .attr('ry', (d) => (d.ry));

// Add x-axis label
svg
  .append("text")
  .attr("x", width / 2 )
  .attr("y", 420 )
  .style("text-anchor", "middle")
  .style("font-style","italic")
  .style("font-weight","bold")
  .text("Petal Length");

// Add y-axis label
svg
  .append("text")
  .attr('transform', 'rotate(-90)')
  .attr('x',0 - (height /2))
  .attr("y",  height/10)
  .style("text-anchor", "middle")
  .style("font-style","italic")
  .style("font-weight","bold")
  .text("Sepal Length");

// Define an array of column names for the data
const columns = [
  'petal_width',
  'sepal_width',
  'petal_length',
  'sepal_length',
];

// Define colors for each label
const labelColors = {
  'setosa': '#01796f',
  'versicolor': '#000080',
  'virginica': '#ff4500'
};

// Add legend
const legend = svg.append('g')
  .attr('transform', `translate(${width - 70},${height -140})`);

// Define legend data
const legendData = ['setosa', 'versicolor', 'virginica'];

// Bind the legend data to the SVG and add legend items with circles and text with specified properties
const legendItem = legend.selectAll('g')
  .data(legendData)
  .join('g')
  .attr('transform', (d, i) => `translate(0,${i * 15})`);

legendItem.append('circle')
  .attr('cx', 10) // set the x-coordinate of the circle center
  .attr('cy', 10) // set the y-coordinate of the circle center
  .attr('r', 5) // set the radius of the circle
  .style('fill', marker); // set the fill color of the circle using the "marker" function

legendItem.append('text')
  .attr('x', 20) // set the x-coordinate of the text
  .attr('y', 10) // set the y-coordinate of the text
  .text((d) => d) // set the text content to the label
  .style('font-size', '12px') // set the font size of the text
  .style('font-weight', 'bold') // set the font weight of the text
  .style('fill', (d) => labelColors[d]) // set the fill color of the text based on the label
  .attr('alignment-baseline', 'middle'); // set the vertical alignment of the text to the middle of the text box

let i = 0;
setInterval(() => {
  plot.xValue((d) => d[columns[i % columns.length]]); // update the x-value function of the scatter plot to cycle through the columns
  svg.call(plot); // call the scatter plot with the updated x-value function
  i++; // increment the column index
}, 2500); // repeat every 2.5 seconds

};
main(); // call the main function to start the visualization




