import {
  scaleLinear, // import scaleLinear function from d3 library
  extent, // import extent function from d3 library
  axisLeft, // import axisLeft function from d3 library
  axisBottom, // import axisBottom function from d3 library
} from 'd3'; // import the entire d3 library and extract specific functions

export const scatterPlot = () => {
  let width; // declare a variable to store the width of the plot
  let height; // declare a variable to store the height of the plot
  let data; // declare a variable to store the data for the plot
  let xValue; // declare a variable to store the x value accessor function
  let yValue; // declare a variable to store the y value accessor function
  let margin; // declare a variable to store the margin object
  let radius; // declare a variable to store the radius of the circles in the plot

  const my = (selection) => {
    // define the my function, which takes a selection of DOM elements as an argument
    const x = scaleLinear() // create a linear scale for the x-axis
      .domain(extent(data, xValue)) // set the domain of the x scale to the extent of the x values in the data
      .range([margin.left, width - margin.right]); // set the range of the x scale to the width of the plot minus the margins

    const y = scaleLinear() // create a linear scale for the y-axis
      .domain(extent(data, yValue)) // set the domain of the y scale to the extent of the y values in the data
      .range([height - margin.bottom, margin.top]); // set the range of the y scale to the height of the plot minus the margins

    const marks = data.map((d) => ({
      x: x(xValue(d)), // map each data point to an object with x and y values based on the scales and accessor functions
      y: y(yValue(d)),
    }));

    const circles = selection // select all circles in the plot
      .selectAll('circle')
      .data(marks) // bind the marks data to the circles
      .join('circle') // create new circles for any new data points
      .attr('cx', (d) => d.x) // set the x position of each circle based on the x value of the corresponding data point
      .attr('cy', (d) => d.y) // set the y position of each circle based on the y value of the corresponding data point
      .attr('r', radius) // set the radius of each circle
      .attr('stroke', '#fcf75e') // set the color of the stroke of each circle
      .attr('stroke-width', .5); // set the width of the stroke of each circle

    selection // select the y-axis group element
      .selectAll('.y-axis')
      .data([null]) // bind a single null value to the y-axis group element
      .join('g') // create a new y-axis group element if one doesn't exist
      .attr('class', 'y-axis') // set the class of the y-axis group element
      .attr('transform', `translate(${margin.left},0)`) // translate the y-axis to the left margin
      .call(axisLeft(y)); // render the y-axis using the axisLeft function

selection
  .selectAll('.x-axis')     // select all elements with class 'x-axis'
  .data([null])             // bind an array with a single null element to the selection
  .join('g')                // create a new <g> element for each null data element, if needed
  .attr('class', 'x-axis')  // add class 'x-axis' to the <g> element
  .attr(
    'transform',
    `translate(0,${height - margin.bottom})`  // set the translation of the <g> element to the bottom of the chart
  )
  .call(axisBottom(x));    // call the axisBottom function with the x scale and apply it to the <g> element
};

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
